#ifndef TIMER_H_
#define TIMER_H_
#include <pthread.h>
#include "PriorityQueue.h"
#include "Queue_TS.h"
#include "Message.h"

struct Timer 
{
	struct Queue* psMsgQ;
	struct PQueue *psPq;
	int alarmID;
	pthread_mutex_t alarmIDMutex;
	pthread_t thread;
	int runThread;
};


// TimerThreadStart()
// call this function to start the timer threads
void TimerThreadStart();

// TimerThreadStop()
// call this function to stop the timer threads
void TimerThreadStop();


// createAlarm()
// This function creates an alarm.
//
// expireTime is the time in milliseconds until the alarm should ring
// psRespQ is the Message Queue where the ALARM_EVENT message will be placed
// repeat is a flag that allows this alarm to repeat automatically every
// 	exprireTime milliseconds (1 = turn on repeat, 0 = turn off repeat).
//
// returns the alarmID of the new alarm, or -1 on error
int createAlarm(int expireTime, struct Queue *psRespQ, int repeat);

// cancelAlarm()
//
// Cancel the alarm denoted by alarmID
// The user may still receive an ALARM_EVENT for this alarm!
void cancelAlarm(int alarmID);

#endif /*TIMER_H_*/
