#include "Timer.h"
#include <pthread.h>
#include <stdio.h>
#include <unistd.h>

#define ALARM_ONESEC 1000

#define USLEEP_QUARTERSEC 250000

// readFromQ
// This function takes a pointer to a Queue as an argument and
// reads 5 items off the queue and then terminates
void* readFromQ(void* arg)
{
	struct Queue *psMyQ = (struct Queue*) arg;
	struct Message *psTmp;
	int len;
	int run = 5;
	
	while(run)
	{
		if( queueProbe(psMyQ))
		{
			// we know we added 5 items to the queue so wait forever
			psTmp = (struct Message *) queueDequeue(psMyQ, &len, QUEUE_BLOCK);
			if( psTmp )
			{
				// we are on the only thread taking data off this Queue but we
				// want to be very careful.
				fprintf(stderr," \t\t\tDEQUEUE %d\n",	psTmp->data.alarm.alarmID);
				free(psTmp);
			}
			run --;
		}
		usleep(USLEEP_QUARTERSEC);
	}
	fprintf(stderr," readFromQ is exiting\n");

	return NULL;
}

int main()
{
	struct Queue *psMyQ = queueInit();
	pthread_t thread;
	pthread_mutex_t testM;

	pthread_mutex_init(&testM, NULL);
	pthread_mutex_lock(&testM);

	// start the timer thread
	TimerThreadStart();
	
	// launch my own thread
	pthread_create(&thread, NULL, readFromQ, (void*)psMyQ);

	// create 5 alarms of varying duration
	createAlarm(1*ALARM_ONESEC, psMyQ, 0);
	createAlarm(2*ALARM_ONESEC, psMyQ, 0);
	createAlarm(3*ALARM_ONESEC, psMyQ, 0);
	sleep(10);
	createAlarm(20*ALARM_ONESEC, psMyQ, 0);
	createAlarm(10*ALARM_ONESEC, psMyQ, 0);

	// wait for my thread to exit
	pthread_join(thread, NULL);
	
	// stop the timer thread
	TimerThreadStop();
	
	return 0;
}


